package spirit;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdesktop.application.Task;
import spirit.database.bean.SqlserverUrl;

/**
 * 合区处理
 * @author 石卓林
 */
public class MergeRunBox extends JDialog {

    private static final Log log = LogFactory.getLog(MergeRunBox.class);
    private SqlserverUrl fromaccount;
    private SqlserverUrl fromgame;
    private SqlserverUrl toaccount;
    private SqlserverUrl togame;

    public MergeRunBox(java.awt.Frame parent, boolean modal, SqlserverUrl fromaccount, SqlserverUrl fromgame, SqlserverUrl toaccount, SqlserverUrl togame) {
        super(parent, modal);
        this.fromaccount = fromaccount;
        this.fromgame = fromgame;
        this.toaccount = toaccount;
        this.togame = togame;
        initComponents();
        new RunTask(org.jdesktop.application.Application.getInstance(spirit.SpiritApp.class), this).execute();
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        progressBar = new javax.swing.JProgressBar();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        progressText = new javax.swing.JLabel();
        progressMain = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(spirit.SpiritApp.class).getContext().getResourceMap(MergeRunBox.class);
        setTitle(resourceMap.getString("Form.title")); // NOI18N
        setName("Form"); // NOI18N
        setResizable(false);

        progressBar.setName("progressBar"); // NOI18N

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        progressText.setText(resourceMap.getString("progressText.text")); // NOI18N
        progressText.setName("progressText"); // NOI18N

        progressMain.setText(resourceMap.getString("progressMain.text")); // NOI18N
        progressMain.setName("progressMain"); // NOI18N

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 405, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(layout.createSequentialGroup()
                        .add(jLabel1)
                        .add(2, 2, 2)
                        .add(progressMain)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 360, Short.MAX_VALUE))
                    .add(layout.createSequentialGroup()
                        .add(jLabel2)
                        .add(2, 2, 2)
                        .add(progressText, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 360, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 24, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(18, 18, 18)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(progressMain))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(progressText))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private class RunTask extends Task<Object, Void> {

        protected JDialog dialog;
        protected Map<String, String> uidmap = new ConcurrentHashMap<String, String>();
        protected Map<String, String> actermap = new ConcurrentHashMap<String, String>();
        protected String[] ext = "abcdefghijklmnopqrstuvwxyz".split("");

        RunTask(org.jdesktop.application.Application app, JDialog dialog) {
            super(app);
            this.dialog = dialog;
        }

        @Override
        protected Object doInBackground() {
            progressBar.setIndeterminate(true);
            msgMainText("正在备份数据库...(1/9)");
            try {
                msgProgressText("备份来源账号库...");
                fromaccount.backup();
                msgProgressText("备份来源游戏库...");
                fromgame.backup();
                msgProgressText("备份目的账号库...");
                toaccount.backup();
                msgProgressText("备份目的游戏库...");
                togame.backup();
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "备份操作失败!");
                return null;
            } finally {
                progressBar.setIndeterminate(false);
            }
            msgMainText("正在传输账号...(2/9)");
            transportAccount();
            msgMainText("正在传输账号资料...(3/9)");
            transportAccountadd();
            msgMainText("正在传输角色...(4/9)");
            transportActer();
            msgMainText("正在传输装备...(5/9)");
            transportItem();
            msgMainText("正在传输魔法...(6/9)");
            transportMagic();
            msgMainText("正在结交朋友...(7/9)");
            transportFriend();
            msgMainText("正在传输结婚...(8/9)");
            transportRelation();
            msgMainText("正在传输任务...(8/9)");
            transportQuest();
            JOptionPane.showMessageDialog(rootPane, "游戏数据合并完成");
            dialog.dispose();
            return null;
        }

        @Override
        protected void succeeded(Object result) {
            JOptionPane.showMessageDialog(rootPane, "处理完成!");
            dialog.dispose();
        }

        protected void transportQuest(){
            Connection conn = null;
            PreparedStatement pstmt = null;
            ResultSet rs = null;
            Connection toConn = null;
            PreparedStatement toPstmt = null;
            try {
                conn = fromgame.getConnection();
                toConn = togame.getConnection();
                pstmt = conn.prepareStatement("SELECT [FLD_CHARACTER],[FLD_QUESTOPENINDEX],[FLD_QUESTFININDEX],[FLD_QUEST] FROM [TBL_QUEST]", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                toPstmt = toConn.prepareStatement("INSERT INTO [TBL_QUEST]([FLD_CHARACTER],[FLD_QUESTOPENINDEX],[FLD_QUESTFININDEX],[FLD_QUEST]) VALUES(?,?,?,?)", ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
                rs = pstmt.executeQuery();
                rs.last();
                int total = rs.getRow();
                if (rs.first()) {
                    do {
                        msgProgressText("任务标签:" + rs.getRow() + "/" + total);
                        msgProgressValue(rs.getRow(), total);
                        String acter = actermap.get(rs.getString("FLD_CHARACTER").trim());
                        if (acter == null) acter = rs.getString("FLD_CHARACTER").trim();
                        toPstmt.setString(1, acter);
                        for (int i = 2; i <= 4; i++) toPstmt.setObject(i, rs.getObject(i));
                        toPstmt.executeUpdate();
                    } while (rs.next());
                }
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "任务数据合并失败");
                log.error("任务数据合并失败", e);
                dialog.dispose();
            } finally {
                closeRs(rs);
                closePstmt(pstmt);
                closePstmt(toPstmt);
                closeConn(conn);
                closeConn(toConn);
            }
        }
        protected void transportRelation() {
            Connection conn = null;
            PreparedStatement pstmt = null;
            ResultSet rs = null;
            Connection toConn = null;
            PreparedStatement toPstmt = null;
            try {
                conn = fromgame.getConnection();
                toConn = togame.getConnection();
                pstmt = conn.prepareStatement("SELECT [FLD_CHARACTER],[FLD_OTHER],[FLD_STATE],[FLD_MSG],[FLD_DATE] FROM [TBL_RELATION]", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                toPstmt = toConn.prepareStatement("INSERT INTO [TBL_RELATION]([FLD_CHARACTER],[FLD_OTHER],[FLD_STATE],[FLD_MSG],[FLD_DATE]) VALUES(?,?,?,?,?)", ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
                rs = pstmt.executeQuery();
                rs.last();
                int total = rs.getRow();
                if (rs.first()) {
                    do {
                        msgProgressText("婚姻关系:" + rs.getRow() + "/" + total);
                        msgProgressValue(rs.getRow(), total);
                        String acter = actermap.get(rs.getString("FLD_CHARACTER").trim());
                        String other = actermap.get(rs.getString("FLD_OTHER").trim());
                        if (acter == null) acter = rs.getString("FLD_CHARACTER").trim();
                        if (other == null) other = rs.getString("FLD_OTHER").trim();
                        toPstmt.setString(1, acter);
                        toPstmt.setString(2, other);
                        for (int i = 3; i <= 5; i++) toPstmt.setObject(i, rs.getObject(i));
                        toPstmt.executeUpdate();
                    } while (rs.next());
                }
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "婚姻数据合并失败");
                log.error("婚姻数据合并失败", e);
                dialog.dispose();
            } finally {
                closeRs(rs);
                closePstmt(pstmt);
                closePstmt(toPstmt);
                closeConn(conn);
                closeConn(toConn);
            }
        }

        protected void transportFriend() {
            Connection conn = null;
            PreparedStatement pstmt = null;
            ResultSet rs = null;
            Connection toConn = null;
            PreparedStatement toPstmt = null;
            try {
                conn = fromgame.getConnection();
                toConn = togame.getConnection();
                pstmt = conn.prepareStatement("SELECT [FLD_CHARACTER],[FLD_FRIEND],[FLD_STATE],[FLD_DESC] FROM [TBL_FRIEND]", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                toPstmt = toConn.prepareStatement("INSERT INTO [TBL_FRIEND]([FLD_CHARACTER],[FLD_FRIEND],[FLD_STATE],[FLD_DESC]) VALUES(?,?,?,?)", ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
                rs = pstmt.executeQuery();
                rs.last();
                int total = rs.getRow();
                if (rs.first()) {
                    do {
                        msgProgressText("朋友关系:" + rs.getRow() + "/" + total);
                        msgProgressValue(rs.getRow(), total);
                        String acter = actermap.get(rs.getString("FLD_CHARACTER").trim());
                        String friend = actermap.get(rs.getString("FLD_FRIEND").trim());
                        if (acter == null) acter = rs.getString("FLD_CHARACTER").trim();
                        if (friend == null) friend = rs.getString("FLD_FRIEND").trim();
                        toPstmt.setString(1, acter);
                        toPstmt.setString(2, friend);
                        for (int i = 3; i <= 4; i++) toPstmt.setObject(i, rs.getObject(i));
                        toPstmt.executeUpdate();
                    } while (rs.next());
                }
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "朋友数据合并失败");
                log.error("朋友数据合并失败", e);
                dialog.dispose();
            } finally {
                closeRs(rs);
                closePstmt(pstmt);
                closePstmt(toPstmt);
                closeConn(conn);
                closeConn(toConn);
            }
        }

        protected void transportMagic() {
            Connection conn = null;
            PreparedStatement pstmt = null;
            ResultSet rs = null;
            Connection toConn = null;
            PreparedStatement toPstmt = null;
            try {
                conn = fromgame.getConnection();
                toConn = togame.getConnection();
                pstmt = conn.prepareStatement("SELECT [FLD_CHARACTER],[FLD_MAGICID],[FLD_POS],[FLD_LEVEL],[FLD_KEY],[FLD_CURTRAIN] FROM [TBL_MAGIC]", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                toPstmt = toConn.prepareStatement("INSERT INTO [TBL_MAGIC]([FLD_CHARACTER],[FLD_MAGICID],[FLD_POS],[FLD_LEVEL],[FLD_KEY],[FLD_CURTRAIN]) VALUES(?,?,?,?,?,?)", ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
                rs = pstmt.executeQuery();
                rs.last();
                int total = rs.getRow();
                if (rs.first()) {
                    do {
                        msgProgressText("传授魔法:" + rs.getRow() + "/" + total);
                        msgProgressValue(rs.getRow(), total);
                        String acter = actermap.get(rs.getString("FLD_CHARACTER").trim());
                        if (acter == null) acter = rs.getString("FLD_CHARACTER").trim();
                        toPstmt.setString(1, acter);
                        for (int i = 2; i <= 6; i++) toPstmt.setObject(i, rs.getObject(i));
                        toPstmt.executeUpdate();
                    } while (rs.next());
                }
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "魔法数据合并失败");
                log.error("魔法数据合并失败", e);
                dialog.dispose();
            } finally {
                closeRs(rs);
                closePstmt(pstmt);
                closePstmt(toPstmt);
                closeConn(conn);
                closeConn(toConn);
            }
        }

        protected void transportItem() {
            //获取装备列表
            Connection conn = null;
            Connection toConn = null;
            PreparedStatement pstmt = null;
            PreparedStatement toPstmt = null;
            ResultSet rs = null;
            try {
                conn = fromgame.getConnection();
                toConn = togame.getConnection();
                pstmt = conn.prepareStatement("SELECT FLD_CHARACTER,FLD_TYPE,FLD_POS,FLD_PRICE,FLD_MAKEINDEX,FLD_INDEX,FLD_DURA,FLD_DURAMAX,FLD_DESC0,FLD_DESC1,FLD_DESC2,FLD_DESC3,FLD_DESC4,FLD_DESC5,FLD_DESC6,FLD_DESC7,FLD_DESC8,FLD_DESC9,FLD_DESC10,FLD_DESC11,FLD_DESC12,FLD_DESC13,FLD_DESC14,FLD_DESC15,FLD_DESC16,FLD_ADDDESC0,FLD_ADDDESC1,FLD_ADDDESC2,FLD_ADDDESC3,FLD_ADDDESC4,FLD_ADDDESC5,FLD_ADDDESC6,FLD_ADDDESC7,FLD_GEM0,FLD_GEM1,FLD_GEM2,FLD_GEM3,FLD_GEM4 FROM TBL_ITEM", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                toPstmt = toConn.prepareStatement("INSERT INTO TBL_ITEM(FLD_CHARACTER,FLD_TYPE,FLD_POS,FLD_PRICE,FLD_MAKEINDEX,FLD_INDEX,FLD_DURA,FLD_DURAMAX,FLD_DESC0,FLD_DESC1,FLD_DESC2,FLD_DESC3,FLD_DESC4,FLD_DESC5,FLD_DESC6,FLD_DESC7,FLD_DESC8,FLD_DESC9,FLD_DESC10,FLD_DESC11,FLD_DESC12,FLD_DESC13,FLD_DESC14,FLD_DESC15,FLD_DESC16,FLD_ADDDESC0,FLD_ADDDESC1,FLD_ADDDESC2,FLD_ADDDESC3,FLD_ADDDESC4,FLD_ADDDESC5,FLD_ADDDESC6,FLD_ADDDESC7,FLD_GEM0,FLD_GEM1,FLD_GEM2,FLD_GEM3,FLD_GEM4) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)", ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
                rs = pstmt.executeQuery();
                rs.last();
                int total = rs.getRow();
                if (rs.first()) {
                    do {
                        msgProgressText("保存装备:" + rs.getRow() + "/" + total);
                        msgProgressValue(rs.getRow(), total);
                        for (int i = 2; i <= 38; i++) toPstmt.setObject(i, rs.getObject(i));
                        String acter = actermap.get(rs.getString("FLD_CHARACTER").trim());
                        if (acter == null) acter = rs.getString("FLD_CHARACTER").trim();
                        toPstmt.setString(1, acter);
                        toPstmt.setInt(5, transportItemId(toConn));
                        toPstmt.executeUpdate();
                    } while (rs.next());
                }
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "装备数据合并失败");
                log.error("装备数据合并失败", e);
                dialog.dispose();
            } finally {
                closePstmt(toPstmt);
                closeRs(rs);
                closePstmt(pstmt);
                closeConn(toConn);
                closeConn(conn);
            }
        }

        protected int transportItemId(Connection conn) {
            //返回最大 id + 1
            PreparedStatement pstmt = null;
            ResultSet rs = null;
            int result = 0;
            try {
                pstmt = conn.prepareStatement("SELECT MAX(FLD_MAKEINDEX) FROM TBL_ITEM");
                rs = pstmt.executeQuery();
                rs.next();
                result = rs.getInt(1);
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "获取装备空闲ID失败");
                log.error("在获取装备空闲ID的时候发生错误", e);
                dialog.dispose();
            } finally {
                closePstmt(pstmt);
                closeRs(rs);
            }
            return result + 1;
        }

        protected void transportActer() {
            //获取角色列表
            Connection conn = null;
            Connection toConn = null;
            PreparedStatement pstmt = null;
            PreparedStatement toPstmt = null;
            ResultSet rs = null;
            try {
                conn = fromgame.getConnection();
                toConn = togame.getConnection();
                pstmt = conn.prepareStatement("SELECT FLD_CHARACTER,FLD_USERID,FLD_DELETED,FLD_UPDATEDATETIME,FLD_DBVERSION,FLD_MAPNAME,FLD_CX,FLD_CY,FLD_DIR,FLD_HAIR,FLD_SEX,FLD_JOB,FLD_LEVEL,FLD_HP,FLD_MP,FLD_EXP,FLD_GOLD,FLD_GAMEGOLD,FLD_HOMEMAP,FLD_HOMEX,FLD_HOMEY,FLD_PKPOINT,FLD_ALLOWPARTY,FLD_FREEGULITYCOUNT,FLD_ATTACKMODE,FLD_FIGHTZONEDIE,FLD_BODYLUCK,FLD_INCHEALTH,FLD_INCSPELL,FLD_INCHEALING,FLD_BONUSAPPLY,FLD_BONUSPOINT,FLD_HUNGRYSTATE,FLD_TESTSERVERRESETCOUNT,FLD_CGHUSETIME,FLD_ENABLEGRECALL,FLD_BYTES_1,FLD_HORSERACE,FLD_MAKEDATE,FLD_FAMECUR,FLD_FAMEBASE,FLD_HUMEXPRATE,FLD_HUMEXPRATETIME,FLD_RELEVEL,FLD_GAMEPOINT FROM TBL_CHARACTER", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                toPstmt = toConn.prepareStatement("INSERT INTO TBL_CHARACTER(FLD_CHARACTER,FLD_USERID,FLD_DELETED,FLD_UPDATEDATETIME,FLD_DBVERSION,FLD_MAPNAME,FLD_CX,FLD_CY,FLD_DIR,FLD_HAIR,FLD_SEX,FLD_JOB,FLD_LEVEL,FLD_HP,FLD_MP,FLD_EXP,FLD_GOLD,FLD_GAMEGOLD,FLD_HOMEMAP,FLD_HOMEX,FLD_HOMEY,FLD_PKPOINT,FLD_ALLOWPARTY,FLD_FREEGULITYCOUNT,FLD_ATTACKMODE,FLD_FIGHTZONEDIE,FLD_BODYLUCK,FLD_INCHEALTH,FLD_INCSPELL,FLD_INCHEALING,FLD_BONUSAPPLY,FLD_BONUSPOINT,FLD_HUNGRYSTATE,FLD_TESTSERVERRESETCOUNT,FLD_CGHUSETIME,FLD_ENABLEGRECALL,FLD_BYTES_1,FLD_HORSERACE,FLD_MAKEDATE,FLD_FAMECUR,FLD_FAMEBASE,FLD_HUMEXPRATE,FLD_HUMEXPRATETIME,FLD_RELEVEL,FLD_GAMEPOINT) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)", ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
                rs = pstmt.executeQuery();
                rs.last();
                int total = rs.getRow();
                if (rs.first()) {
                    do {
                        //获取角色名称如果角色名称需要改变则存储改变映射
                        int count = 1;
                        String acter = rs.getString("FLD_CHARACTER").trim();
                        while (transportActerCount(acter, toConn) != 0) {
                            //短于14位末尾加数字
                            if (acter.replaceAll("[\\u4e00-\\u9fa5]", "uu").length() < 14 && count == 1) {
                                acter += ext[count];
                            } else {
                                //否则最后一位替换为数字
                                acter = acter.substring(0, acter.length() - ext[count].length()) + ext[count];
                            }
                            count++;
                        }
                        //如果被改变记录 key原始acter val新acter
                        if (!acter.equals(rs.getString("FLD_CHARACTER").trim())) {
                            actermap.put(rs.getString("FLD_CHARACTER").trim(), acter);
                            log.info("角色: " + rs.getString("FLD_CHARACTER").trim() + " 更名为 " + acter);
                        }
                        String uid = uidmap.get(rs.getString("FLD_USERID").trim());
                        if (uid == null) uid = rs.getString("FLD_USERID").trim();
                        //存入角色信息
                        msgProgressText("保存角色:" + acter + " 原名:" + rs.getString("FLD_CHARACTER"));
                        msgProgressValue(rs.getRow(), total);
                        toPstmt.setString(1, acter);
                        toPstmt.setString(2, uid);
                        for (int i = 3; i <= 45; i++) toPstmt.setObject(i, rs.getObject(i));
                        toPstmt.executeUpdate();
                    } while (rs.next());
                }
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "角色数据合并失败");
                log.error("角色数据合并失败", e);
                dialog.dispose();
            } finally {
                closeConn(toConn);
                closeRs(rs);
                closePstmt(toPstmt);
                closePstmt(pstmt);
                closeConn(conn);
            }
        }

        protected int transportActerCount(String acter, Connection conn) {
            PreparedStatement pstmt = null;
            ResultSet rs = null;
            int result = 0;
            try {
                pstmt = conn.prepareStatement("SELECT COUNT(*) FROM TBL_CHARACTER WHERE FLD_CHARACTER = ?");
                pstmt.setString(1, acter);
                rs = pstmt.executeQuery();
                rs.next();
                result = rs.getInt(1);
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "角色数据合并失败");
                log.error("在判断角色是否重复的时候发生错误", e);
                dialog.dispose();
            } finally {
                closePstmt(pstmt);
                closeRs(rs);
            }
            return result;
        }

        protected void transportAccountadd() {
            //获取账号资料来源列表
            Connection conn = null;
            Connection toConn = null;
            PreparedStatement pstmt = null;
            PreparedStatement toPstmt = null;
            ResultSet rs = null;
            try {
                conn = fromaccount.getConnection();
                toConn = toaccount.getConnection();
                pstmt = conn.prepareStatement("SELECT FLD_LOGINID,FLD_SSNO,FLD_BIRTHDAY,FLD_PHONE,FLD_MOBILEPHONE,FLD_ADDRESS1,FLD_ADDRESS2,FLD_EMAIL,FLD_QUIZ1,FLD_ANSWER1,FLD_QUIZ2,FLD_ANSWER2,FLD_ZIPCODE,FLD_ACCEPTNEWS,FLD_MON,FLD_PCHECK,FLD_PTEL,FLD_PSSNO,FLD_PNAME,FLD_PEMAIL,FLD_USERNAME FROM TBL_ACCOUNTADD", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                toPstmt = toConn.prepareStatement("INSERT INTO TBL_ACCOUNTADD(FLD_LOGINID,FLD_SSNO,FLD_BIRTHDAY,FLD_PHONE,FLD_MOBILEPHONE,FLD_ADDRESS1,FLD_ADDRESS2,FLD_EMAIL,FLD_QUIZ1,FLD_ANSWER1,FLD_QUIZ2,FLD_ANSWER2,FLD_ZIPCODE,FLD_ACCEPTNEWS,FLD_MON,FLD_PCHECK,FLD_PTEL,FLD_PSSNO,FLD_PNAME,FLD_PEMAIL,FLD_USERNAME) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)", ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
                rs = pstmt.executeQuery();
                rs.last();
                int total = rs.getRow();
                if (rs.first()) {
                    do {
                        String uid = uidmap.get(rs.getString("FLD_LOGINID").trim());
                        if (uid == null) uid = rs.getString("FLD_LOGINID").trim();
                        msgProgressText("保存账号资料:" + uid);
                        msgProgressValue(rs.getRow(), total);
                        toPstmt.setString(1, uid);
                        for (int i = 2; i <= 21; i++) toPstmt.setObject(i, rs.getObject(i));
                        toPstmt.executeUpdate();
                    } while (rs.next());
                }
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "账号资料数据合并失败");
                log.error("账号资料数据合并失败", e);
                dialog.dispose();
            } finally {
                closeConn(toConn);
                closePstmt(toPstmt);
                closeRs(rs);
                closePstmt(pstmt);
                closeConn(conn);
            }
        }

        protected void transportAccount() {
            //获取账号来源列表
            Connection conn = null;
            Connection toConn = null;
            PreparedStatement pstmt = null;
            PreparedStatement toPstmt = null;
            ResultSet rs = null;
            try {
                conn = fromaccount.getConnection();
                toConn = toaccount.getConnection();
                pstmt = conn.prepareStatement("SELECT FLD_LOGINID,FLD_PASSWORD,FLD_PASSFAILCOUNT,FLD_PASSFAILTIME,FLD_MAKETIME,FLD_VER,FLD_VALIDFROM,FLD_M2VALIDFROM,FLD_M3VALIDFROM,FLD_VALIDUNTIL,FLD_FreeVALIDUNTIL,FLD_M2VALIDUNTIL,FLD_FreeM2VALIDUNTIL,FLD_M3VALIDUNTIL,FLD_FreeM3VALIDUNTIL,FLD_SECONDS,FLD_FreeSECONDS,FLD_M2SECONDS,FLD_FreeM2SECONDS,FLD_M3SECONDS,FLD_FreeM3SECONDS,FLD_STOPUNTIL,FLD_M2STOPUNTIL,FLD_M3STOPUNTIL,FLD_SSNO,FLD_PCHECK,FLD_SECEDE,FLD_SERVERIP FROM TBL_ACCOUNT", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                toPstmt = toConn.prepareStatement("INSERT INTO TBL_ACCOUNT(FLD_LOGINID,FLD_PASSWORD,FLD_PASSFAILCOUNT,FLD_PASSFAILTIME,FLD_MAKETIME,FLD_VER,FLD_VALIDFROM,FLD_M2VALIDFROM,FLD_M3VALIDFROM,FLD_VALIDUNTIL,FLD_FreeVALIDUNTIL,FLD_M2VALIDUNTIL,FLD_FreeM2VALIDUNTIL,FLD_M3VALIDUNTIL,FLD_FreeM3VALIDUNTIL,FLD_SECONDS,FLD_FreeSECONDS,FLD_M2SECONDS,FLD_FreeM2SECONDS,FLD_M3SECONDS,FLD_FreeM3SECONDS,FLD_STOPUNTIL,FLD_M2STOPUNTIL,FLD_M3STOPUNTIL,FLD_SSNO,FLD_PCHECK,FLD_SECEDE,FLD_SERVERIP) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)", ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
                rs = pstmt.executeQuery();
                rs.last();
                int total = rs.getRow();
                if (rs.first()) {
                    do {
                        //判断账号是否重复
                        int count = 1;
                        String uid = rs.getString("FLD_LOGINID").trim();
                        while (transportAccountCount(uid, toConn) != 0) {
                            //短于十位末尾加数字
                            if (uid.length() < 10 && count == 1) {
                                uid += ext[count];
                            } else {
                                //否则最后一位替换为数字
                                uid = uid.substring(0, uid.length() - ext[count].length()) + ext[count];
                            }
                            count++;
                        }
                        //如果被改变记录患春存 简明,原始uid val新uid
                        if (!uid.equals(rs.getString("FLD_LOGINID").trim())) {
                            uidmap.put(rs.getString("FLD_LOGINID").trim(), uid);
                            log.info("账号:" + rs.getString("FLD_LOGINID").trim() + " 更名为 " + uid);
                        }
                        //存入账号信息
                        msgProgressText("保存账号:" + uid + " 原名:" + rs.getString("FLD_LOGINID"));
                        msgProgressValue(rs.getRow(), total);
                        toPstmt.setString(1, uid);
                        for (int i = 2; i <= 28; i++) toPstmt.setObject(i, rs.getObject(i));
                        toPstmt.executeUpdate();
                    } while (rs.next());
                }
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "账号数据合并失败");
                log.error("账号数据合并失败", e);
                dialog.dispose();
            } finally {
                closeConn(toConn);
                closeConn(conn);
                closePstmt(toPstmt);
                closePstmt(pstmt);
                closeRs(rs);
            }
        }

        protected int transportAccountCount(String uid, Connection conn) {
            PreparedStatement pstmt = null;
            ResultSet rs = null;
            int result = 0;
            try {
                pstmt = conn.prepareStatement("SELECT COUNT(*) FROM TBL_ACCOUNT WHERE FLD_LOGINID = ?");
                pstmt.setString(1, uid);
                rs = pstmt.executeQuery();
                rs.next();
                result = rs.getInt(1);
            } catch (SQLException e) {
                JOptionPane.showMessageDialog(rootPane, "账号数据合并失败");
                log.error("在判断账号是否重复的时候发生错误", e);
                dialog.dispose();
            } finally {
                closePstmt(pstmt);
                closeRs(rs);
            }
            return result;
        }
        /* 关闭资源 start */

        protected void closeConn(Connection conn) {
            try {
                conn.close();
            } catch (Exception e) {
            }
        }

        protected void closePstmt(PreparedStatement pstmt) {
            try {
                pstmt.close();
            } catch (Exception e) {
            }
        }

        protected void closeStmt(Statement stmt) {
            try {
                stmt.close();
            } catch (Exception e) {
            }
        }

        protected void closeRs(ResultSet rs) {
            try {
                rs.close();
            } catch (Exception e) {
            }
        }
        /* 关闭资源 end */
        /* 消息输出 start */

        protected void msgMainText(String msg) {
            SwingUtilities.invokeLater(new msgMainText(msg));
        }

        protected class msgMainText implements Runnable {

            private String msg;

            public msgMainText(String msg) {
                this.msg = msg;
            }

            public void run() {
                progressMain.setText(msg);
            }
        }

        protected void msgProgressText(String msg) {
            SwingUtilities.invokeLater(new msgProgressText(msg));
        }

        protected class msgProgressText implements Runnable {

            private String msg;

            public msgProgressText(String msg) {
                this.msg = msg;
            }

            public void run() {
                progressText.setText(msg);
            }
        }

        protected void msgProgressValue(int current, int total) {
            SwingUtilities.invokeLater(new msgProgressValue(current, total));
        }

        protected class msgProgressValue implements Runnable {

            private int value;

            public msgProgressValue(int current, int total) {
                value = Math.round((((float) current) / ((float) total)) * 100);
            }

            public void run() {
                progressBar.setValue(value);
            }
        }
        /* 消息输入 end */
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel progressMain;
    private javax.swing.JLabel progressText;
    // End of variables declaration//GEN-END:variables
}
